<?php

namespace FiloBlu\Job\Model\Mail;

use Magento\Framework\App\TemplateTypesInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Mail\EmailMessageInterfaceFactory;
use Magento\Framework\Mail\MessageInterfaceFactory;
use Magento\Framework\Mail\MimeInterface;
use Magento\Framework\Mail\Template\FactoryInterface;
use Magento\Framework\Mail\Template\SenderResolverInterface;
use Magento\Framework\Mail\TransportInterfaceFactory;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Phrase;
use Zend\Mime\MessageFactory as MimeMessageFactory;
use Zend\Mime\Mime;
use Zend\Mime\PartFactory;

/**
 * Class TransportBuilder
 * @package FiloBlu\Job\Model\Mail
 */
class TransportBuilder extends \Magento\Framework\Mail\Template\TransportBuilder
{
    /**
     * @var array
     */
    protected $attachments = [];
    /**
     * @var PartFactory
     */
    protected $partFactory;
    /**
     * @var MimeMessageFactory
     */
    protected $mimeMessageFactory;

    /**
     * TransportBuilder constructor.
     * @param FactoryInterface $templateFactory
     * @param SenderResolverInterface $senderResolver
     * @param ObjectManagerInterface $objectManager
     * @param TransportInterfaceFactory $mailTransportFactory
     * @param PartFactory $partFactory
     * @param MimeMessageFactory $mimeMessageFactory
     * @param MessageInterfaceFactory $messageFactory
     */
    public function __construct(
        FactoryInterface $templateFactory,
        SenderResolverInterface $senderResolver,
        ObjectManagerInterface $objectManager,
        TransportInterfaceFactory $mailTransportFactory,
        PartFactory $partFactory,
        MimeMessageFactory $mimeMessageFactory,
        MessageInterfaceFactory $messageFactory = null
    )
    {
        $message = $messageFactory->create();
        parent::__construct($templateFactory, $message, $senderResolver, $objectManager, $mailTransportFactory, $messageFactory);
        $this->partFactory = $partFactory;
        $this->mimeMessageFactory = $mimeMessageFactory;
    }

    /**
     * Add an attachment to the message.
     *
     * @param string $file
     * @param string $fileName
     * @return $this
     */
    public function addAttachment($file, $fileName)
    {
        $data = file_get_contents($file);
        $mime = 'application/octet-stream';

        if (function_exists('finfo_open')) {
            $finfo = finfo_open(FILEINFO_MIME);
            $mime = finfo_buffer($finfo, $data, FILEINFO_MIME);
            finfo_close($finfo);
        } elseif (function_exists('mime_content_type')) {
            $mime = mime_content_type($file);
        }

        if ($this->message !== null) {
            $this->message->setBodyAttachment($data, $fileName, $mime);
            return $this;
        }


        $this->attachments[] = [
            'content' => $data,
            'type' => $mime,
            'disposition' => Mime::DISPOSITION_ATTACHMENT,
            'encoding' => Mime::ENCODING_BASE64
        ];

        return $this;
    }

    /**
     * After all parts are set, add them to message body.
     *
     * @return $this
     * @throws LocalizedException
     */
    protected function prepareMessage()
    {
        // Magento > 2.3.3
        if ($this->message === null) {
            return $this->prepareMessageAter233();
        }

        return $this->prepareMessageBefore232();

    }

    /**
     * @return $this
     * @throws LocalizedException
     */
    protected function prepareMessageAter233()
    {
        parent::prepareMessage();

        $template = $this->getTemplate();
        $content = $template->processTemplate();

        switch ($template->getType()) {
            case TemplateTypesInterface::TYPE_TEXT:
                $part['type'] = MimeInterface::TYPE_TEXT;
                break;

            case TemplateTypesInterface::TYPE_HTML:
                $part['type'] = MimeInterface::TYPE_HTML;
                break;

            default:
                throw new LocalizedException(
                    new Phrase('Unknown template type')
                );
        }
        $mimePart = $this->objectManager->create(\Magento\Framework\Mail\MimePartInterface::class, ['content' => $content]);
        $attachment = $this->objectManager->create(\Magento\Framework\Mail\MimePartInterface::class, array_shift($this->attachments));
        $mimeMessageInterfaceFactory = $this->objectManager->create(\Magento\Framework\Mail\MimeMessageInterfaceFactory::class);

        $to = $this->message->getTo();
        $messageData['to'] = $to;
        $messageData['from'] = $this->message->getFrom();
        $messageData['cc'] = $this->message->getCc();
        $messageData['bcc'] = $this->message->getBcc();
        $messageData['replyTo'] = $this->message->getReplyTo();
        $messageData['sender'] = $this->message->getSender();
        $messageData['encoding'] = $mimePart->getCharset();
        $messageData['body'] = $mimeMessageInterfaceFactory->create(['parts' => [$mimePart, $attachment]]);
        $messageData['subject'] = html_entity_decode((string)$template->getSubject(), ENT_QUOTES);

        $emailMessageInterfaceFactory = $this->objectManager->get(EmailMessageInterfaceFactory::class);
        $this->message = $emailMessageInterfaceFactory->create($messageData);

        return $this;
    }

    /**
     * @return $this
     * @throws LocalizedException
     */
    protected function prepareMessageBefore232()
    {
        $template = $this->getTemplate();
        $body = $template->processTemplate();

        switch ($template->getType()) {
            case TemplateTypesInterface::TYPE_TEXT:
                $this->message->setBodyText($body);
                break;

            case TemplateTypesInterface::TYPE_HTML:
                $this->message->setBodyHtml($body);
                break;

            default:
                throw new LocalizedException(
                    new Phrase('Unknown template type')
                );
        }
        $this->message->setPartsToBody();
        $this->message->setSubject(html_entity_decode($template->getSubject(), ENT_QUOTES));
        return $this;
    }
}